"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseClient = void 0;
const Constants_1 = require("../utils/Constants");
const axios_1 = require("axios");
const https_1 = require("https");
const eventemitter3_1 = require("eventemitter3");
const url_1 = require("url");
const Methods_1 = require("../utils/Methods");
const ErrorHandler_1 = require("../errors/ErrorHandler");
/**
 * Base Client for the Client to extend
 * @extends EventEmitter
 */
class BaseClient extends eventemitter3_1.EventEmitter {
    constructor(options = {}, payload = {}) {
        super();
        this._bearer = null;
        // Set defaults
        this._basic = options.basic || (0, Methods_1.createBasicAuth)();
        this._bearer = options.bearer || null;
        const headers = options?.headers ?? Constants_1.DefaultHeaders;
        headers["user-agent"] = options?.userAgent ?? Constants_1.DefaultUserAgent;
        this.instance = axios_1.default.create({
            baseURL: `https://${Constants_1.APIBase}/v${Constants_1.APIVersion}`,
            headers: headers,
            httpsAgent: new https_1.Agent({ keepAlive: true }),
        });
        this.instance.interceptors.request.use((config) => {
            config.headers ?? (config.headers = headers);
            config.headers.Authorization = this.authorization;
            //console.log(`set headers:\n ${JSON.stringify(config.headers, null, 2)}`);
            return config;
        });
        this.instance.interceptors.response.use((onSuccess) => onSuccess, (onFail) => {
            const error = (0, ErrorHandler_1.handleError)(onFail);
            if (error instanceof Error)
                throw error;
            return error;
        });
        this._payload = Object.assign({}, payload);
    }
    /**
     * The authorization string used for requests
     */
    get authorization() {
        return this._bearer ? `Bearer ${this._bearer}` : `Basic ${this._basic}`;
    }
    /**
     * Gets the payload
     */
    get payload() {
        return this._payload;
    }
    set payload(value) {
        this._payload = Object.assign(this.payload, value);
    }
    /**
     * Calls eval with the client set as the "this" context.
     * @param script The script to evaluate
     * @returns The result of the script
     */
    _eval(script) {
        return eval(script);
    }
    /**
     * Paginates through items in a REST API endpoint.
     * @param url The url to get
     * @param key The key to get the array of values from
     * @param params The params to pass to the url
     * @yields The value of the key
     */
    async *paginate(url, key, params) {
        if (!url)
            throw new Error("paginate: url is required");
        if (!key)
            throw new Error("paginate: key is required");
        let method = "GET";
        if (url.toLowerCase() === "collective") {
            method = "POST";
        }
        let hasNext;
        if (!params)
            params = new url_1.URLSearchParams();
        let config = method === "GET"
            ? { method, url, params: new url_1.URLSearchParams(params) }
            : { method, url, data: params || {} };
        do {
            let { data } = await this.instance.request(config);
            const items = data?.data?.[key]?.items;
            if (!Array.isArray(items))
                throw new Error("paginate: items is not an array");
            hasNext = data?.data?.[key]?.paging?.has_next ?? false;
            if (hasNext) {
                params.set("next", data.data[key].paging.cursors.next);
            }
            for (let item of items) {
                yield item;
            }
        } while (hasNext);
    }
}
exports.BaseClient = BaseClient;
//# sourceMappingURL=BaseClient.js.map