"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const BaseClient_1 = require("./BaseClient");
const ifunny_api_types_1 = require("@ifunny/ifunny-api-types");
const UserManager_1 = require("../managers/UserManager");
const iFunnyError_1 = require("../errors/iFunnyError");
const iFunnyErrorCodes_1 = require("../errors/iFunnyErrorCodes");
/**
 * Client for the Discord API.
 * @extends BaseClient
 */
class Client extends BaseClient_1.BaseClient {
    constructor(config, payload = {}) {
        super(config, payload);
        this._users = new UserManager_1.UserManager(this);
    }
    get basic() {
        return this._basic;
    }
    set basic(value) {
        this._basic = value;
    }
    /**
     * The bearer token for the client
     */
    get bearer() {
        return this._bearer;
    }
    /**
     * Sets the bearer token for the client. If set to null, the client will not use a bearer token.
     */
    set bearer(value) {
        this._bearer = value || null;
        this.instance.defaults.headers.common.Authorization = this.isAuthorized()
            ? `Bearer ${this.bearer}`
            : `Basic ${this.basic}`;
    }
    /**
     * Is the client using a bearer token to make requests?
     */
    isAuthorized() {
        return !!this.bearer;
    }
    /**
     * Updates the Client's payload
     * @returns The Client instance
     */
    async fetch() {
        if (!this.isAuthorized()) {
            throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.ClientNotLoggedIn, "Fetch account data");
        }
        const response = await this.instance.get(ifunny_api_types_1.Endpoints.account);
        let data = response.data;
        this.payload = data.data;
        return this;
    }
    /**
     * The client's user manager
     */
    get users() {
        return this._users;
    }
    /**
     * Logs into iFunny with an Email and Password\
     * If you already have a bearer token, don't use this method
     * @param email The email to login with
     * @param password The password to login with
     * @returns The instance of the client
     */
    async login(email, password) {
        if (this.isAuthorized()) {
            return await this.fetch();
        }
        let response = await this.instance.post(ifunny_api_types_1.Endpoints.token, {
            grant_type: "password",
            username: email,
            password,
        });
        this.bearer = response.data.access_token;
        return await this.fetch();
    }
    /**
     * Signs up for an iFunny account.
     * @param nick The nick of the account to sign up with. Checks if the nick is taken.
     * @param email The email to sign up with. Checks if the email is taken.
     * @param password The password to sign up with
     * @param login Should the client login after sign up?
     * @returns The instance of the client
     */
    async signUp(nick, email, password, acceptMailOffers = false, login = true) {
        let response = await this.instance.post(ifunny_api_types_1.Endpoints.user(), {
            reg_type: "pwd",
            nick,
            email,
            password,
            accept_mailing: acceptMailOffers ? 1 : 0,
        });
        this.payload.id = response.data.data.id;
        if (login) {
            return await this.login(email, password);
        }
        return this;
    }
}
exports.Client = Client;
exports.default = Client;
//# sourceMappingURL=Client.js.map