"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBasicAuth = exports.sleep = exports.dateFromTimestamp = exports.isAxiosError = exports.assert = exports.ifModifiedSince = void 0;
const crypto = require("crypto");
const axios_1 = require("axios");
const Constants_1 = require("./Constants");
const iFunnyError_1 = require("../errors/iFunnyError");
const iFunnyErrorCodes_1 = require("../errors/iFunnyErrorCodes");
/**
 * Certain requests have a header for 'if-modified-since' which uses the current timestamp
 * @param date Custom date
 * @returns GMT timestamp of the date
 */
function ifModifiedSince(date) {
    date ?? (date = new Date());
    return date.toUTCString();
}
exports.ifModifiedSince = ifModifiedSince;
/**
 * Asserts a claim to be true
 * @param claim The claim to check
 * @param error The message to send if the claim evaluted to false
 */
function assert(claim, error) {
    if (!claim) {
        if (typeof error === "string") {
            throw new Error(error);
        }
        else if (error instanceof Error) {
            throw error;
        }
    }
    return true;
}
exports.assert = assert;
/**
 * Validates that something is an Axios Error
 * @param item Item to validate
 */
function isAxiosError(item) {
    return axios_1.default.isAxiosError(item);
}
exports.isAxiosError = isAxiosError;
/**
 * Returns a Date object from a timestamp
 * @param timestamp Number of seconds of the timestamp
 * @param inMilliseconds Whether the timestamp is already in milliseconds or not
 * @returns
 */
function dateFromTimestamp(timestamp, inMilliseconds = true) {
    // ? Converts timestamp to milliseconds if necessary
    if (!inMilliseconds)
        timestamp *= 1000;
    return new Date(timestamp);
}
exports.dateFromTimestamp = dateFromTimestamp;
/**
 * Sleeps the program for the specified amount of milliseconds
 * @param ms The number of milliseconds to wait
 * @returns A promise that resolves after the given amount of time
 */
async function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
exports.sleep = sleep;
/**
 * Generates a basic token
 * @param opts Options to create the token with
 * @param opts.clientId The client id to use for the token
 * @param opts.clientSecret The client secret to use for the token
 * @param opts.length The length of the token to create
 * @returns
 */
function createBasicAuth(opts) {
    const { length, clientId, clientSecret } = Object.assign(Constants_1.DefaultBasicAuthConfig, opts);
    let uuid = crypto.randomUUID().replace(/\-/g, "");
    let hex;
    switch (length) {
        case 112:
            hex = uuid.toUpperCase();
            break;
        case 156:
            hex = crypto.createHash("sha256").update(uuid).digest("hex").toUpperCase();
            break;
        default:
            throw new iFunnyError_1.iFunnyError(iFunnyErrorCodes_1.iFunnyErrorCodes.InvalidBasicTokenLength);
    }
    let a = hex + `_${clientId}:`;
    let b = hex + `:${clientId}:${clientSecret}`;
    let c = crypto.createHash("sha1").update(b).digest("hex");
    return Buffer.from(a + c).toString("base64");
}
exports.createBasicAuth = createBasicAuth;
//# sourceMappingURL=Methods.js.map